def d(x):
    from sklearn import datasets
    from sklearn.cross_validation import cross_val_predict
    from sklearn import linear_model
    lr = linear_model.LinearRegression()
    boston = datasets.load_boston()
    y = boston.target

    # cross_val_predict returns an array of the same size as `y` where each entry
    # is a prediction obtained by cross validated:
    predicted = cross_val_predict(lr, boston.data, y, cv=10)
    return predicted
